@@ -21,7 +21,7 @@
 
 #include "config.h"
 #include <stdio.h>
-#include <malloc.h>
+#include <stdlib.h>
 #include <errno.h>
 #include <string.h>
 #include "trace.h"
