@@ -456,10 +456,9 @@ static int ini_aug_construct_list(char *dirname ,
             }
         }
         else {
-            ini_aug_add_string(ra_err,
-                               "File %s did not match provided patterns."
-                               " Skipping.",
-                               fullname);
+            TRACE_INFO_STRING("File did not match provided patterns."
+                              " Skipping:",
+                              fullname);
         }
     }
 
@@ -609,8 +608,8 @@ static int ini_aug_match_sec(struct ini_cfgobj *snip_cfg,
 
     /* Just in case check that we processed anything */
     if (section_count == 0) {
-        ini_aug_add_string(ra_err, "No sections found in file %s. Skipping.",
-                           snip_name);
+        TRACE_INFO_STRING("No sections found in file. Skipping:",
+                          snip_name);
         *skip = true;
         TRACE_FLOW_EXIT();
         return EOK;
