@@ -29,8 +29,20 @@
 #include "collection.h"
 #include "collection_tools.h"
 #include "ini_defines.h"
-#include "ini_config_priv.h"
 #include "ini_configobj.h"
+#include "ini_config_priv.h"
+
+/* Following declarations are from header file ini_config.h. This file was not
+ * included, because we don't want on include header file
+ * with old interface(ini_config.h) and new interface(ini_configobj.h)
+ * into the one file.
+ */
+void print_config_parsing_errors(FILE *file,
+                                 struct collection_item *error_set);
+
+
+void print_file_parsing_errors(FILE *file,
+                               struct collection_item *error_list);
 
 
 /*============================================================*/
@@ -126,7 +138,9 @@ static const char *parsing_error_str(int parsing_error)
                                 _("Invalid character at the "
                                   "beginning of the line."),
                                 _("Invalid tab character at the "
-                                  "beginning of the line.")
+                                  "beginning of the line."),
+                                _("Incomplete comment at the "
+                                  "end of the file.")
     };
 
     /* Check the range */
@@ -152,7 +166,7 @@ static const char *parsing_error_str(int parsing_error)
  * This error is returned when the template
  * is translated into the grammar object.
  *
- * @param[in] parsing_error    Error code for the grammar error.
+ * @param[in] grammar_error    Error code for the grammar error.
  *
  * @return Error string.
  */
@@ -194,7 +208,7 @@ static const char *grammar_error_str(int grammar_error)
  * the INI file is validated against the
  * grammar object.
  *
- * @param[in] parsing_error    Error code for the validation error.
+ * @param[in] validation_error    Error code for the validation error.
  *
  * @return Error string.
  */
@@ -336,11 +350,13 @@ void print_file_parsing_errors(FILE *file,
 }
 
 
+void print_grammar_errors(FILE *file,
+                          struct collection_item *error_list);
 /* Print errors and warnings that were detected while processing grammar.
  *
  * The following doxygen description is moved here.
  * When the function gets exposed move it into
- * the header file.
+ * the header file and remove prototype from this file.
  */
 /**
  * @brief Print errors and warnings that were detected while
@@ -365,11 +381,13 @@ void print_grammar_errors(FILE *file,
                      INI_FAMILY_GRAMMAR);
 }
 
+void print_validation_errors(FILE *file,
+                             struct collection_item *error_list);
 /* Print errors and warnings that were detected while validating INI file.
  *
  * The following doxygen description is moved here.
  * When the function gets exposed move it into
- * the header file.
+ * the header file and remove prototype from this file.
  */
 /**
  * @brief Print errors and warnings that were detected while
